@@ -36,3 +36,11 @@ class DeferredInstanceException(Exception):
     """
 
     pass
+
+
+class AggregatorException(Exception):
+    pass
+
+
+class GridSearchException(Exception):
+    pass
\ No newline at end of file
