@@ -1,25 +1,26 @@
 import copy
 import inspect
 from functools import wraps
+from numbers import Number
 from random import random
 from typing import Tuple, Optional
 
 import numpy as np
 
-import autofit.mapper.model
-import autofit.mapper.model_mapper
-import autofit.mapper.prior_model.collection
-from autofit import cast_collection, PriorNameValue, InstanceNameValue
 from autofit import exc
+from autofit.mapper import model
+from autofit.mapper import model_mapper
 from autofit.mapper.model import AbstractModel
+from autofit.mapper.prior.deferred import DeferredArgument
+from autofit.mapper.prior.prior import GaussianPrior
+from autofit.mapper.prior.prior import TuplePrior, Prior, WidthModifier, Limits
+from autofit.mapper.prior_model import collection
 from autofit.mapper.prior_model import dimension_type as dim
 from autofit.mapper.prior_model.attribute_pair import DeferredNameValue
-from autofit.mapper.prior_model.deferred import DeferredArgument
-from autofit.mapper.prior_model.prior import GaussianPrior
-from autofit.mapper.prior_model.prior import TuplePrior, Prior, WidthModifier, Limits
+from autofit.mapper.prior_model.attribute_pair import cast_collection, PriorNameValue, InstanceNameValue
 from autofit.mapper.prior_model.recursion import DynamicRecursionCache
 from autofit.mapper.prior_model.util import PriorModelNameValue
-from autofit.tools.text_formatter import TextFormatter
+from autofit.text.formatter import TextFormatter
 
 
 def check_assertions(func):
@@ -56,7 +57,6 @@ class AbstractPriorModel(AbstractModel):
     Abstract model that maps a set of priors to a particular class. Must be
     overridden by any prior model so that the model mapper recognises its prior \
     model attributes.
-
     @DynamicAttrs
     """
 
@@ -69,7 +69,6 @@ class AbstractPriorModel(AbstractModel):
         Assert that some relationship holds between physical values associated with
         priors at the point an instance is created. If this fails a FitException is
         raised causing the model to be re-sampled.
-
         Parameters
         ----------
         assertion
@@ -99,31 +98,27 @@ class AbstractPriorModel(AbstractModel):
             obj.__init__(t, **kwargs)
         elif isinstance(t, list) or isinstance(t, dict):
             obj = object.__new__(
-                autofit.mapper.prior_model.collection.CollectionPriorModel
+                collection.CollectionPriorModel
             )
             obj.__init__(t)
         else:
             obj = t
         return obj
 
-    def instance_from_unit_vector(self, unit_vector):
+    def instance_from_unit_vector(self, unit_vector, assert_priors_in_limits=True):
         """
         Creates a ModelInstance, which has an attribute and class instance corresponding
         to every PriorModel attributed to this instance.
-
         This method takes as input a unit vector of parameter values, converting each to
         physical values via their priors.
-
         Parameters
         ----------
         unit_vector: [float]
-            A vector of physical parameter values.
-
+            A unit hypercube vector that is mapped to an instance of physical values via the priors.
         Returns
         -------
         model_instance : autofit.mapper.model.ModelInstance
             An object containing reconstructed model_mapper instances
-
         Raises
         ------
         exc.FitException
@@ -140,7 +135,7 @@ class AbstractPriorModel(AbstractModel):
             )
         )
 
-        return self.instance_for_arguments(arguments)
+        return self.instance_for_arguments(arguments, assert_priors_in_limits=assert_priors_in_limits)
 
     @property
     @cast_collection(PriorNameValue)
@@ -158,7 +153,7 @@ class AbstractPriorModel(AbstractModel):
 
     @property
     def unique_promise_tuples(self):
-        from autofit import AbstractPromise
+        from autofit.mapper.prior.promise import AbstractPromise
 
         return {
             prior_tuple[1]: prior_tuple
@@ -184,7 +179,6 @@ class AbstractPriorModel(AbstractModel):
         ----------
         unit_vector: [float]
             A unit hypercube vector
-
         Returns
         -------
         values: [float]
@@ -198,9 +192,20 @@ class AbstractPriorModel(AbstractModel):
             )
         )
 
-    @property
-    def random_vector_from_priors(self):
+    def random_unit_vector_within_limits(self, lower_limit=0.0, upper_limit=1.0):
+        """ Generate a random vector of unit values by drawing uniform random values between 0 and 1.
+        Returns
+        -------
+        unit_values: [float]
+            A list of unit values constructed by taking random values from each prior.
         """
+        return list(np.random.uniform(low=lower_limit, high=upper_limit, size=self.prior_count))
+
+    def random_vector_from_priors_within_limits(self, lower_limit, upper_limit):
+        """ Generate a random vector of physical values by drawing uniform random values between an input lower and
+        upper limit and using the model priors to map them from unit values to physical values.
+        This is used for MCMC initialization, whereby the starting points of a walker(s) is confined to a restricted
+        range of prior space. In particular, it is used for generate the "ball" initialization of Emcee.
         Returns
         -------
         physical_values: [float]
@@ -211,7 +216,7 @@ class AbstractPriorModel(AbstractModel):
         while True:
 
             vector = self.vector_from_unit_vector(
-                list(np.random.random(self.prior_count))
+                list(np.random.uniform(low=lower_limit, high=upper_limit, size=self.prior_count))
             )
 
             try:
@@ -220,6 +225,17 @@ class AbstractPriorModel(AbstractModel):
             except exc.PriorLimitException:
                 pass
 
+    @property
+    def random_vector_from_priors(self):
+        """ Generate a random vector of physical values by drawing uniform random values between 0 and 1 and using
+        the model priors to map them from unit values to physical values.
+        Returns
+        -------
+        physical_values: [float]
+            A list of physical values constructed by taking random values from each prior.
+        """
+        return self.random_vector_from_priors_within_limits(lower_limit=0.0, upper_limit=1.0)
+
     @property
     def physical_values_from_prior_medians(self):
         """
@@ -231,24 +247,26 @@ class AbstractPriorModel(AbstractModel):
         """
         return self.vector_from_unit_vector([0.5] * len(self.unique_prior_tuples))
 
-    def instance_from_vector(self, vector):
+    def instance_from_vector(
+            self,
+            vector,
+            assert_priors_in_limits=True
+    ):
         """
         Creates a ModelInstance, which has an attribute and class instance corresponding
         to every PriorModel attributed to this instance.
-
         This method takes as input a physical vector of parameter values, thus omitting
         the use of priors.
-
         Parameters
         ----------
         vector: [float]
-            A unit hypercube vector
-
+            A vector of physical parameter values that is mapped to an instance.
+        assert_priors_in_limits
+            If True it is checked that the physical values of priors are within set limits
         Returns
         -------
         model_instance : autofit.mapper.model.ModelInstance
             An object containing reconstructed model_mapper instances
-
         """
         arguments = dict(
             map(
@@ -258,18 +276,19 @@ class AbstractPriorModel(AbstractModel):
             )
         )
 
-        return self.instance_for_arguments(arguments)
+        return self.instance_for_arguments(
+            arguments,
+            assert_priors_in_limits=assert_priors_in_limits
+        )
 
     def mapper_from_partial_prior_arguments(self, arguments):
         """
         Creates a new model mapper from a dictionary mapping_matrix existing priors to
         new priors, keeping existing priors where no mapping is provided.
-
         Parameters
         ----------
         arguments: {Prior: Prior}
             A dictionary mapping_matrix priors to priors
-
         Returns
         -------
         model_mapper: ModelMapper
@@ -282,12 +301,10 @@ class AbstractPriorModel(AbstractModel):
         """
         Creates a new model mapper from a dictionary mapping_matrix existing priors to
         new priors.
-
         Parameters
         ----------
         arguments: {Prior: Prior}
             A dictionary mapping_matrix priors to priors
-
         Returns
         -------
         model_mapper: ModelMapper
@@ -318,11 +335,8 @@ class AbstractPriorModel(AbstractModel):
         of gaussian priors. The widths of the new priors are taken from the
         width_config. The new gaussian priors must be provided in the same order as
         the priors associated with model.
-
         If a is not None then all priors are created with an absolute width of a.
-
         If r is not None then all priors are created with a relative width of r.
-
         Parameters
         ----------
         no_limits
@@ -333,7 +347,6 @@ class AbstractPriorModel(AbstractModel):
             The absolute width to be assigned to gaussian priors
         tuples
             A list of tuples each containing the mean and width of a prior
-
         Returns
         -------
         mapper: ModelMapper
@@ -390,17 +403,39 @@ class AbstractPriorModel(AbstractModel):
     def instance_from_prior_medians(self):
         """
         Creates a list of physical values from the median values of the priors.
-
         Returns
         -------
         physical_values : [float]
             A list of physical values
-
         """
         return self.instance_from_unit_vector(
             unit_vector=[0.5] * len(self.prior_tuples)
         )
 
+    def log_priors_from_vector(
+            self,
+            vector: [float],
+    ):
+        """
+        Compute the log priors of every parameter in a vector, using the Prior of every parameter.
+        The log prior values are used by Emcee to map the log likelihood to the poserior of the model.
+        Parameters
+        ----------
+        vector : [float]
+            A vector of physical parameter values.
+        Returns
+        -------
+        log_priors : []
+            An list of the log prior value of every parameter.
+        """
+        return list(
+            map(
+                lambda prior_tuple, value: prior_tuple.prior.log_prior_from_value(value=value),
+                self.prior_tuples_ordered_by_id,
+                vector,
+            )
+        )
+
     def random_instance(self):
         """
         Creates a random instance of the model.
@@ -414,13 +449,11 @@ class AbstractPriorModel(AbstractModel):
     def from_instance(instance, model_classes=tuple()):
         """
         Recursively create an prior object model from an object model.
-
         Parameters
         ----------
         model_classes
         instance
             A dictionary, list, class instance or model instance
-
         Returns
         -------
         abstract_prior_model
@@ -428,14 +461,14 @@ class AbstractPriorModel(AbstractModel):
         """
 
         if isinstance(instance, list):
-            result = autofit.mapper.prior_model.collection.CollectionPriorModel(
+            result = collection.CollectionPriorModel(
                 [
                     AbstractPriorModel.from_instance(item, model_classes=model_classes)
                     for item in instance
                 ]
             )
-        elif isinstance(instance, autofit.mapper.model.ModelInstance):
-            result = autofit.mapper.model_mapper.ModelMapper()
+        elif isinstance(instance, model.ModelInstance):
+            result = model_mapper.ModelMapper()
             for key, value in instance.dict.items():
                 setattr(
                     result,
@@ -445,7 +478,7 @@ class AbstractPriorModel(AbstractModel):
                     ),
                 )
         elif isinstance(instance, dict):
-            result = autofit.mapper.prior_model.collection.CollectionPriorModel(
+            result = collection.CollectionPriorModel(
                 {
                     key: AbstractPriorModel.from_instance(
                         value, model_classes=model_classes
@@ -557,9 +590,39 @@ class AbstractPriorModel(AbstractModel):
                 d.update(prior_model[1].prior_class_dict)
         return d
 
-    def instance_for_arguments(self, arguments):
+    def _instance_for_arguments(self, arguments):
         raise NotImplementedError()
 
+    def instance_for_arguments(
+            self,
+            arguments,
+            assert_priors_in_limits=True
+    ):
+        """
+        Create an instance of the model for a set of arguments
+        Parameters
+        ----------
+        assert_priors_in_limits
+            If true it is asserted that the physical values that replace piors are
+            within their limits
+        arguments: {Prior: float}
+            Dictionary mapping_matrix priors to attribute analysis_path and value pairs
+        Returns
+        -------
+            An instance of the class
+        """
+        if self.promise_count > 0:
+            raise exc.PriorException(
+                "All promises must be populated prior to instantiation"
+            )
+        if assert_priors_in_limits:
+            for prior, value in arguments.items():
+                if isinstance(value, Number):
+                    prior.assert_within_limits(value)
+        return self._instance_for_arguments(
+            arguments
+        )
+
     @property
     def prior_count(self):
         return len(self.unique_prior_tuples)
@@ -613,7 +676,6 @@ class AbstractPriorModel(AbstractModel):
         """
         Recursively overwrite priors in the mapper with instance values from the
         instance except where the containing class is the descendant of a listed class.
-
         Parameters
         ----------
         excluded_classes
@@ -633,14 +695,11 @@ class AbstractPriorModel(AbstractModel):
     def path_for_prior(self, prior: Prior) -> Optional[Tuple[str]]:
         """
         Find a path that points at the given tuple.
-
         Returns the first path or None if no path is found.
-
         Parameters
         ----------
         prior
             A prior representing what's known about some dimension of the model.
-
         Returns
         -------
         A path, a series of attributes that point to one location of the prior.
@@ -676,31 +735,34 @@ class AbstractPriorModel(AbstractModel):
         }
 
     @property
-    def info(self):
+    def info(self) -> str:
         """
         Use the priors that make up the model_mapper to generate information on each
         parameter of the overall model.
-
         This information is extracted from each priors *model_info* property.
         """
+        from autofit.mapper.prior import AbstractPromise
         formatter = TextFormatter()
 
-        for t in self.path_priors_tuples + self.path_float_tuples:
+        for t in self.path_instance_tuples_for_class((
+                Prior, float, AbstractPromise, tuple
+        )):
             formatter.add(t)
 
         return formatter.text
 
     @property
-    def param_names(self):
+    def parameter_names(self) -> [str]:
         """The param_names vector is a list each parameter's analysis_path, and is used
         for *GetDist* visualization.
-
         The parameter names are determined from the class instance names of the
         model_mapper. Latex tags are properties of each model class."""
-
         return [
-            self.name_for_prior(prior)
-            for prior in sorted(self.priors, key=lambda prior: prior.id)
+            self.name_for_prior(
+                prior
+            )
+            for _, prior
+            in self.prior_tuples_ordered_by_id
         ]
 
 
@@ -708,7 +770,6 @@ def transfer_classes(instance, mapper, model_classes=None):
     """
     Recursively overwrite priors in the mapper with instance values from the
     instance except where the containing class is the descendant of a listed class.
-
     Parameters
     ----------
     model_classes
